/*
** Module   : ClientBuilderParameters.java
** Abstract : store the process parameters.
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- -------------------------------Description------------------------------------
** 001 MAG 20131119 First version.
** 002 MAG 20140206 Add referrer property.
** 003 CA  20140206 Refactored to make it unaware of the type of the P2J client being spawned.
*/

package com.goldencode.p2j.main;

import java.util.*;

/**
 * This structure holds default parameters as well specific parameters used to create and spawn a
 * P2J client process. 
 */
public class ClientBuilderParameters
{   
   /** Default options from directory */
   protected final ClientBuilderOptions options;

   /** Client assigned uuid */
   private String uuid;

   /** User name */
   private String userName;
   
   /** Password */
   private String password;

   /**
    * Constructor. Assign a unique UUID to this process.
    */
   public ClientBuilderParameters(ClientBuilderOptions options)
   {
      this.options = options;
      uuid = UUID.randomUUID().toString();
   }

   /**
    * Get the authentication mode for this spawner.
    * 
    * @return   {@code true} indicating that the user's password is required for authentication.
    */
   public boolean isPasswordAuthentication()
   {
      return options.isPasswordAuthentication();
   }

   /**
    * Get process assigned UUID.
    * 
    * @return  The assigned UUID. 
    */
   public String getUuid()
   {
      return uuid;
   }
   
   /**
    * Get user name.
    * 
    * @return  User name.
    */
   public String getUserName()
   {
      return userName;
   }

   /**
    * Set user name.
    * 
    * @param   userName
    *          User name.
    */
   public void setUserName(String userName)
   {
      this.userName = userName;
   }

   /**
    * Get user password;
    * 
    * @return  User password.
    */
   public String getPassword()
   {
      return password;
   }

   /**
    * Set user password.
    * 
    * @param   password
    *          User password;
    */

   public void setPassword(String password)
   {
      this.password = password;
   }

   /**
    * Get the security mode..
    * 
    * @return  <code>true</code> use a secured connection <code>false</code> otherwise.
    */
   public boolean isSecure()
   {
      return options.isSecure();
   }

   /**
    * Get command
    * 
    * @return  Command name string.
    */
   public String getCommand()
   {
      return options.getCommand();
   }

   /**
    * Get classpath
    * 
    * @return  Class path string.
    */
   public String getClassPath()
   {
      return options.getClassPath();
   }

   /**
    * Get native library path.
    * 
    * @return  Native library path string.
    */
   public String getLibPath()
   {
      return options.getLibPath();
   }

   /**
    * Get user working directory.
    * 
    * @return  User working directory.
    */
   public String getWorkingDirectory()
   {
      return options.getWorkingDirectory();
   }

   /**
    * Get configuration file name.
    * 
    * @return  Configuration file name.
    */
   public String getConfigFile()
   {
      return options.getConfigFile();
   }
   
   /**
    * Get JVM arguments
    * 
    * @return  A List with JVM arguments.
    */
   public List<String> getJvmArguments()
   {
      return options.getJvmArguments();
   }
   
   /**
    * Get P2J client's application arguments.
    * 
    * @return  A List with application arguments.
    */
   public List<String> getAppArguments()
   {
      return options.getAppArguments();
   }
   
   /**
    * Get P2J client's bootstrap config.
    * 
    * @return  A List with bootstrap config settings.
    */
   public List<String> getBootstrapConfig()
   {
      return options.getBootstrapConfig();
   }

   /**
    * Get spawner command.
    * 
    * @return  The spawner command;
    */
   public String getSpawner()
   {
      return options.getSpawner();
   }

   /**
    * Get the Java process main class.
    * 
    * @return  The main class.
    */
   public Class<?> getMainClass()
   {
      return options.getMainClass();
   }
}
