/*
** Module   : ProcessBuilderOptions.java
** Abstract : store P2J process parameters read from directory.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140206 First version.
*/

package com.goldencode.p2j.main;

import com.goldencode.p2j.security.SecurityManager;
import com.goldencode.p2j.util.*;
import com.goldencode.p2j.util.Utils.DirScope;
import com.goldencode.util.*;

/**
 * The parameters stored inside this structure are read from directory on server start-up 
 * and are used as default parameters to create and spawn a web client process.
 * <p>
 * The container node is named {@link #DIRECTORY_NODE_ID} and the first child of these nodes is
 * used:
 * <ol>
 *    <li>server/&lt;server-id&gt;/runtime/&lt;process-id&gt;/</li>
 *    <li>server/&lt;server-id&gt;/runtime/default/</li>
 *    <li>server/&lt;server-id&gt;/</li>
 *    <li>server/default/</li>
 * </ol>
 */
public class ProcessBuilderOptions
extends ClientBuilderOptions
{
   /** Directory node name, {@code "clientConfig"} string. */
   private static final String DIRECTORY_NODE_ID = "clientConfig";

   /** The alias in the process keystore. */
   private final String processAlias;
   
   /** The alias in the truststore. */
   private final String truststoreAlias;
   
   /** The keystore password. */
   private final String keystorePassword;
   
   /** The keyentry password. */
   private final String keyentryPassword;
   
   /** The truststore password. */
   private final String truststorePassword;

   /** Flag indicating if the certificate must be validated. */
   private final boolean validateCertificate;

   /** The serialized process keystore. */
   private final byte[] processKeystore;

   /** The serialized truststore. */
   private final byte[] truststore;
   
   /** All the subject IDs associated with this process. */
   private final String[] accountIds;

   /**
    * Initialize. Read all configuration parameters from directory.
    * 
    * @param    process
    *           The process subject ID.
    */
   public ProcessBuilderOptions(String process)
   {
      super();
      
      SecurityManager sm = SecurityManager.getInstance();
      
      String ksPassword = RandomWordGenerator.create();
      String kePassword = RandomWordGenerator.create();
      String procAlias  = sm.getAccountAlias(process);

      this.accountIds = sm.getAccountIds(process);

      // "security:certificate:validate" and "security:keystore:processalias" are per process
      this.validateCertificate = getNode(DIRECTORY_NODE_ID, "security:certificate:validate",
                                         false);
      this.processKeystore     = sm.getEncryptedKeyStore(procAlias, ksPassword, kePassword);
      if (this.processKeystore == null)
      {
         this.processAlias     = null;
         this.keystorePassword = null;
         this.keyentryPassword = null;
      }
      else
      {
         this.processAlias     = procAlias;
         this.keystorePassword = ksPassword;
         this.keyentryPassword = kePassword;
      }

      this.truststorePassword  = RandomWordGenerator.create();
      this.truststoreAlias     = sm.getServerAlias();
      this.truststore          = sm.getEncryptedTrustStore(truststorePassword);
   }

   /**
    * Getter for the {@link #processAlias}.
    * 
    * @return   See above.
    */
   public String getProcessAlias()
   {
      return processAlias;
   }

   /**
    * Getter for the {@link #truststoreAlias}.
    * 
    * @return   See above.
    */
   public String getTruststoreAlias()
   {
      return truststoreAlias;
   }

   /**
    * Getter for the {@link #keystorePassword}.
    * 
    * @return   See above.
    */
   public String getKeystorePassword()
   {
      return keystorePassword;
   }

   /**
    * Getter for the {@link #keyentryPassword}.
    * 
    * @return   See above.
    */
   public String getKeyentryPassword()
   {
      return keyentryPassword;
   }

   /**
    * Getter for the {@link #truststorePassword}.
    * 
    * @return   See above.
    */
   public String getTruststorePassword()
   {
      return truststorePassword;
   }

   /**
    * Getter for the {@link #validateCertificate}.
    * 
    * @return   See above.
    */
   public boolean isValidateCertificate()
   {
      return validateCertificate;
   }

   /**
    * Getter for the {@link #processKeystore}.
    * 
    * @return   See above.
    */
   public byte[] getProcessKeystore()
   {
      return processKeystore;
   }

   /**
    * Getter for the {@link #truststore}.
    * 
    * @return   See above.
    */
   public byte[] getTruststore()
   {
      return truststore;
   }

   /**
    * Get the authentication mode for this spawner.
    * 
    * @return   Always {@code false}.
    */
   protected boolean isPasswordAuthentication()
   {
      return false;
   }

   /**
    * Get a string value from a directory node.
    *  
    * @param   parent
    *          The name of the parent node.
    * @param   node
    *          Node name.
    * @param   defa
    *          Default value if node not found.
    *          
    * @return  Node value if exists or the default value otherwise.
    */
   protected String getNode(String parent, String node, String defa)
   {
      return Utils.getDirectoryNodeString(null, parent + "/" + node, defa,
                                          DirScope.BOTH, accountIds);
   }

   /**
    * Get an integer value from a directory node.
    * 
    * @param   parent
    *          The name of the parent node.
    * @param   node
    *          Node name.
    * @param   defa
    *          Default value if node not found.
    *          
    * @return  Node value if exists or the default value otherwise.
    */
   protected int getNode(String parent, String node, int defa)
   {
      return Utils.getDirectoryNodeInt(null, parent + "/" + node, defa,
                                       DirScope.BOTH, accountIds);
   }

   /**
    * Get a boolean value from a directory node.
    *  
    * @param   parent
    *          The name of the parent node.
    * @param   node
    *          Node name.
    * @param   defa
    *          Default value if node not found.
    *          
    * @return  Node value if exists or the default value otherwise.
    */
   protected boolean getNode(String parent, String node, boolean defa)
   {
      return Utils.getDirectoryNodeBoolean(null, parent + "/" + node, defa,
                                           DirScope.BOTH, accountIds);
   }

   /**
    * Get a byte array from a directory node.
    *  
    * @param   parent
    *          The name of the parent node.
    * @param   node
    *          Node name.
    * @param   defa
    *          Default value if node not found.
    *          
    * @return  Node value if exists or the default value otherwise.
    */
   protected byte[] getNode(String parent, String node, byte[] defa)
   {
      return Utils.getDirectoryNodeByteArray(null, parent + "/" + node, defa,
                                             DirScope.BOTH, accountIds);
   }
}
