/*
** Module   : ProcessClientBuilder.java
** Abstract : build and start a platform process for P2J process.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140206 First version.
*/
package com.goldencode.p2j.main;

import java.util.*;

import com.goldencode.p2j.cfg.*;
import com.goldencode.p2j.net.*;
import com.goldencode.p2j.security.SecurityManager;

/**
 * This class is used to start P2J client processes using the {@code spawn} tool, with no
 * password authentication. The command line parameters for the spawned process are constructed 
 * from default parameters stored in directory and some specific parameters stored inside a 
 * {@link ClientBuilderParameters} structure.  A {@link ProcessBuilderOptions} instance is used
 * to store specific P2J process configuration.
 * <p>
 * The syntax used to start the native process is 
 * {@code spawn 0 <secure-port> <server-alias> <uuid>}.
 * 
 * @see   ClientBuilder
 *        For details about the native tool used to spawn client processes.
 */
public class ProcessClientBuilder
extends ClientBuilder
{
   /**
    * Constructor.
    * 
    * @param   config
    *          The specific parameters for this client.
    */
   public ProcessClientBuilder(ClientBuilderParameters config)
   {
      super(config);
   }

   /**
    * Mark this P2J process to start in batch mode (add the {@code client:mode:batch=true} 
    * argument to the command).
    */
   protected void addClientOptions()
   {
      command.add("client:mode:batch=true");
   }
   
   /**
    * Get the spawn arguments.  P2J client processes call the {@code spawn} tool using the
    * {@code spawn 0 <secure-port> <server-alias> <uuid>} syntax.
    * 
    * @return   See above.
    */
   @Override
   protected List<String> getSpawnArguments()
   {
      SessionManager sessMgr = SessionManager.get();
      BootstrapConfig cfg    = sessMgr.config();
      int securePort         = cfg.getInt("net", "server", "secure_port", -1);

      SecurityManager sm = SecurityManager.getInstance();
      
      List<String> cmd = new ArrayList<>();

      // add mode
      cmd.add("0");
      // add secure port
      cmd.add(Integer.toString(securePort));
      // add host; this is hard-coded to localhost because is expected that the in-process JVM
      // will connect to the same P2J server which initiated the request (as this is the P2J
      // server knowing the command to be executed).
      cmd.add("localhost");
      // add server alias
      cmd.add(sm.getServerAlias());
      // add UUID
      cmd.add(config.getUuid());

      return cmd;
   }
 }
