/*
** Module   : TemporaryAccountWorker.java
** Abstract : a Runnable object used to manage user accounts.
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131216 Created initial version.
** 002 CA  20140214 Fixed NPE at thread shutdown when the pool is never used. 
*/

package com.goldencode.p2j.main;

import java.util.concurrent.*;
import java.util.logging.*;

import com.goldencode.p2j.util.*;

/**
 * A Runnable object used to manage user accounts.
 * In order to manage user account admin rights are mandatory.
 * This object is used as Runnable target in an AssociatedThread instance
 * which run in standard server security context and inherit server admin rights.
 * This object execute the run method of another Runnable object the task to be executed.
 * The execute method is synchronized between this thread and caller.
 * The caller waits for end of execution of task that he provide as parameter.
 *  
 * @author mag
 *
 */
public class TemporaryAccountWorker 
implements Runnable
{
   /** Logger. */
   private static final Logger LOG = LogHelper.getLogger(TemporaryAccountWorker.class.getName());
   
   /**  Termination flag */
   private volatile boolean terminated = false;
   
   /** Thread synchronization */
   private volatile CountDownLatch workerLatch;

   /** Caller synchronization */
   private volatile CountDownLatch callerLatch;
   
   /** Task to be executed */
   private volatile Runnable task;
   
   /**
    * Constructor. 
    * Initialize thread synchronization latch.
    */
   public TemporaryAccountWorker()
   {
      workerLatch = new CountDownLatch(1);      
   }

   /**
    * Execute specified task.
    * 
    * @param   task
    *          Task to be executed.
    */
   public void execute(Runnable task)
   {
      try
      {
         this.task = task;
         // Arm caller latch.
         callerLatch = new CountDownLatch(1);
         // Call thread.
         workerLatch.countDown();
         // Wait for thread notification.
         callerLatch.await();      
      }
      catch (InterruptedException e)
      {
         LOG.logp(Level.WARNING,
                  "TemporaryAccountWorker.execute()",
                  "",
                  LogHelper.generate("InterruptedException!"),
                  e);         
      }
   }
   
   /**
    * Task executor.
    */
   public void run()
   {
      while (!terminated)
      {
         try
         {
            // Waiting for a call
            workerLatch.await();
            
            if (task != null)
            {
               task.run();
               task = null;
            }
         }
         catch (InterruptedException e)
         {
            LOG.logp(Level.WARNING,
                     "TemporaryAccountWorker.run()",
                     "",
                     LogHelper.generate("InterruptedException!"),
                     e);
         }
         finally
         {
            workerLatch = new CountDownLatch(1);
            if (callerLatch != null)
            {
               // Notify caller
               callerLatch.countDown();
            }
         }
      }
   }

   /**
    * Terminate thread.
    */
   public void terminate()
   {
      this.terminated = true;
      workerLatch.countDown();
   }
}
