/*
** Module   : WebClientSpawner.java
** Abstract : define the Web Client Spawner.
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 MAG 20131119 First version.
** 002 MAG 20140206 Send URL as referrer address via configuration file.
** 003 CA  20140206 Changes related to refactoring the spawner infrastructure (allow P2J process 
**                  clients to be started, beside web chui clients).
*/

package com.goldencode.p2j.main;

import com.goldencode.p2j.cfg.*;
import com.goldencode.p2j.ui.client.chui.driver.web.*;
import com.goldencode.p2j.ui.client.driver.*;

/**
 * Class used to spawn a new web client process.
 * <p>
 * The spawned process first try to authenticate the user against OS.
 * On success a new process (client) is spawned. The spawned process will get the 
 * server KeyStore, starts the client embedded server and finally will notify this 
 * instance about the URI on which the client embedded server listen.
 * If a notification does not occur on an amount of time a failure in spawned 
 * process is asserted.  
 * 
 * @author mag
 *
 */
public class WebClientSpawner
extends ClientSpawner
{
   /** Remote server URI */
   private volatile String remoteUri = null;
   
   /**
    * Create a new web client spawner.
    */
   public WebClientSpawner()
   {
      super(new WebClientBuilderParameters());
   }

   /**
    * Notify when the client has started.
    * 
    * @param    data
    *           Custom data sent by the P2J client back to the server.  For web clients, this is
    *           the remote URI to which it will redirect the browser.
    */
   @Override
   public void clientIsReady(Object data)
   {
      remoteUri = (String) data;
      
      super.clientIsReady(data);
   }

   /**
    * Get the server key store.
    * 
    * @return   See above.
    */
   @Override
   public Object getServerData()
   {
      return ((WebClientBuilder) cb).getServerKeyStore();
   }
   
   /**
    * Get a {@link WebClientBuilder} used to spawn P2J clients.
    * 
    * @param    cfg
    *           The P2J web client configuration.
    *           
    * @return   See above.
    */
   protected ClientBuilder getBuilder(ClientBuilderParameters cfg)
   {
      return new WebClientBuilder(cfg);
   }

   /**
    * Get the remote server URI to which the browser will be redirected.
    * 
    * @return   The remote server URI or <code>null</code>.
    */
   public String getRemoteUri()
   {
      return remoteUri;
   }

   /**
    * Get a {@link TemporaryClient} worker which will do the work after authenticating on the P2J
    * server using the temporary credentials.
    * 
    * @return   See above.
    */
   @Override
   public TemporaryClient getTemporaryClient()
   {
      return new TemporaryClientTask();
   }

   /**
    * On P2J client side, it will do custom work after authenticating using the temporary 
    * credentials.
    * <p>
    * For web chui clients case, it will create a {@link ChuiWebDriver} and start the embedded 
    * web server which will serve this client's requests.
    */
   public static class TemporaryClientTask
   implements TemporaryClient
   {
      /**
       * On P2J client side, it will do custom work after authenticating using the temporary 
       * credentials.
       * <p>
       * For web chui clients case, it will create a {@link ChuiWebDriver} and start the web 
       * embedded server which will serve this client's requests.
       * 
       * @param    spawner
       *           The spawner remote access interface. 
       * @param    uuid
       *           Remote client identifier uuid.
       * @param    config
       *           The configuration to use for setup of the client and for the server connection.
       *
       * @return   An explicit web screen driver to be used by the P2J client.
       * 
       * @throws   RuntimeException
       *           If the embedded web server could not be started.
       */
      @Override
      public ScreenDriver doWork(Spawner spawner, String uuid, BootstrapConfig config)
      {
         // Import server key store
         ServerKeyStore keyStore = (ServerKeyStore) spawner.exportData(uuid);
         if (keyStore == null)
         {
            throw new RuntimeException("Cannot import server key store.");
         }

         // get an explicit port and host set by the client
         int port = config.getInt("client", "web", "port", 0);
         String host = config.getString("client", "web", "host", null);
         
         // Create web screen driver
         ChuiWebDriver driver = new ChuiWebDriver(config);
         driver.init();
         String uri = ((ChuiWebDriver) driver).startupServer(keyStore, host, port);
         
         // Embedded server URI notification 
         spawner.clientIsReady(uuid, uri);
         if (uri == null)
         {
            throw new RuntimeException("Cannot start embedded server."); 
         }
         
         return driver;
      }
  }
}
