/*
** Module   : Job.java
** Abstract : POJO defining a scheduled job.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140206 First version.
*/

package com.goldencode.p2j.scheduler;

import org.quartz.*;

/**
 * Definition of a {@link Scheduler} job.
 * <p>
 * For each job, the following can be defined:
 * <ul>
 *    <li>{@link #name} - the job name; must be unique and is case insensitive.</li>
 *    <li>{@link #type} - the kind of the job being started:
 *       <p> {@link JobType#PROCESS} - start a new P2J process.
 *       <p>-{@link JobType#CLASS} - execute a certain java class which implements 
 *           {@link Runnable}.
 *       <p>-{@link JobType#METHOD} - execute a certain java method.
 *    </li>
 *    <li>{@link #target}
 *       <p>- in case of {@link JobType#PROCESS}, is the subject ID of a P2J process.
 *       <p>- in case of {@link JobType#CLASS}, is the qualified name of a Java class which  
 *            implements the {@link Runnable} interface 
 *       <p>- in case of {@link JobType#METHOD}, is the qualified name of a Java class followed by 
 *            the name of a no-arguments method, i.e. {@code foo.bar.Something.execute}
 *       </p>
 *    </li>
 *    <li>{@link #enabled} - flag indicating if this job is enabled or not.</li>
 *    <li>{@link #mode} - the job mode. One of the following values:
 *       <p>- {@link JobMode#NOW} - for jobs being executed immediately
 *       <p>- {@link JobMode#ONE_TIME} - for jobs being executed at a certain time/date,
 *           a single time.
 *       <p>- {@link JobMode#RECURRENT} - for recurrent jobs executed at a certain time/date.
 *    </li>
 *    <li>The {@link #second}, {@link #minute}, {@link #hour}, {@link #dayOfMonth}, 
 *        {@link #dayOfWeek}, {@link #month} and {@link #year} need to be specified only for 
 *        {@link JobMode#RECURRENT} or {@link JobMode#ONE_TIME} jobs.  They follow the 
 *        {@link CronExpression} specification for each cron field.
 *     </li>
 * </ul>
 */
public class Job
{
   /** The name of this job. */
   private final String name;

   /** The type of this job, one of the {@link JobType} values. */
   private JobType type;

   /** The target of this job. */
   private String target;
   
   /** Flag indicating if this job is enabled. */
   private boolean enabled;
   
   /** The execution mode of this job, one of the {@link JobMode} values. */
   private JobMode mode;

   /** The second expression when the job needs to be executed. */
   private String second;

   /** The minute expression when the job needs to be executed. */
   private String minute;
   
   /** The hour expression when the job needs to be executed. */
   private String hour;
   
   /** The day the month expression when the job needs to be executed. */
   private String dayOfMonth;
   
   /** The day of the week expression when the job needs to be executed. */
   private String dayOfWeek;
   
   /** The month expression when the job needs to be executed. */
   private String month;
   
   /** The year expression when the job needs to be executed. */
   private String year;
   
   /**
    * Create a new job.
    * 
    * @param    name
    *           The name of this job.
    * @param    type
    *           target
    * @param    target
    *           The target of this job.
    * @param    enabled
    *           Flag indicating if this job is enabled.
    * @param    mode
    *           The execution mode of this job, one of the {@link JobMode} values.
    */
   public Job(String name, JobType type, String target, boolean enabled, JobMode mode)
   {
      this.name = name;
      this.type = type;
      this.target = target;
      this.enabled = enabled;
      this.mode = mode;
   }

   /**
    * Get the job's {@link #name}.
    * 
    * @return   See above.
    */
   public String getName()
   {
      return name;
   }

   /**
    * Get the job's {@link #type}.
    * 
    * @return   See above.
    */
   public JobType getType()
   {
      return type;
   }

   /**
    * Get the job's {@link #target}.
    * 
    * @return   See above.
    */
   public String getTarget()
   {
      return target;
   }

   /**
    * Get the job's {@link #enabled} state.
    * 
    * @return   See above.
    */
   public boolean isEnabled()
   {
      return enabled;
   }

   /**
    * Get the job's {@link #mode}.
    * 
    * @return   See above.
    */
   public JobMode getMode()
   {
      return mode;
   }

   /**
    * Get the job's {@link #second}.
    * 
    * @return   See above.
    */
   public String getSecond()
   {
      return second;
   }

   /**
    * Set the job's execution {@link #second}.
    * 
    * @param    second
    *           The second, {@code null} or a value following the {@link CronExpression} syntax.
    */
   public void setSecond(String second)
   {
      this.second = second;
   }

   /**
    * Get the job's {@link #minute}.
    * 
    * @return   See above.
    */
   public String getMinute()
   {
      return minute;
   }

   /**
    * Set the job's execution {@link #minute}.
    * 
    * @param    minute
    *           The minute, {@code null} or a value following the {@link CronExpression} syntax.
    */
   public void setMinute(String minute)
   {
      this.minute = minute;
   }

   /**
    * Get the job's {@link #hour}.
    * 
    * @return   See above.
    */
   public String getHour()
   {
      return hour;
   }

   /**
    * Set the job's execution {@link #hour}.
    * 
    * @param    hour
    *           The hour, {@code null} or a value following the {@link CronExpression} syntax.
    */
   public void setHour(String hour)
   {
      this.hour = hour;
   }

   /**
    * Get the job's {@link #dayOfMonth}.
    * 
    * @return   See above.
    */
   public String getDayOfMonth()
   {
      return dayOfMonth;
   }

   /**
    * Set the job's execution {@link #dayOfMonth}.
    * 
    * @param    dayOfMonth
    *           The day of the month, {@code null} or a value following the {@link CronExpression}
    *           syntax.
    */
   public void setDayOfMonth(String dayOfMonth)
   {
      this.dayOfMonth = dayOfMonth;
   }

   /**
    * Get the job's {@link #dayOfWeek}.
    * 
    * @return   See above.
    */
   public String getDayOfWeek()
   {
      return dayOfWeek;
   }

   /**
    * Set the job's execution {@link #dayOfWeek}.
    * 
    * @param    dayOfWeek
    *           The day of the week, {@code null} or a value following the {@link CronExpression} 
    *           syntax.
    */
   public void setDayOfWeek(String dayOfWeek)
   {
      this.dayOfWeek = dayOfWeek;
   }

   /**
    * Get the job's {@link #month}.
    * 
    * @return   See above.
    */
   public String getMonth()
   {
      return month;
   }

   /**
    * Set the job's execution {@link #month}.
    * 
    * @param    month
    *           The month, {@code null} or a value following the {@link CronExpression} syntax.
    */
   public void setMonth(String month)
   {
      this.month = month;
   }

   /**
    * Get the job's {@link #year}.
    * 
    * @return   See above.
    */
   public String getYear()
   {
      return year;
   }

   /**
    * Set the job's execution {@link #year}.
    * 
    * @param    year
    *           The year, {@code null} or a year following the {@link CronExpression} syntax.
    */
   public void setYear(String year)
   {
      this.year = year;
   }
   
   /**
    * Get the string representation of this job, using the 
    * {@code "<name> [<type>:<mode>] - <target>"} format.
    */
   @Override
   public String toString()
   {
      return String.format("%s [%s:%s] - %s", this.name, this.type, this.mode, this.target);
   }
}
