/*
** Module   : JobMode.java
** Abstract : enum defining the possible job modes.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140206 First version.
*/

package com.goldencode.p2j.scheduler;

/**
 * Definition of possible job modes:
 * <ul>
 * <li>{@link #NOW} - execute immediately.</li>
 * <li>{@link #ONE_TIME} - schedule for execution at a certain time/date.</li>
 * <li>{@link #RECURRENT} - schedule for execution at a certain time/date, recurrent.</li>
 * </ul>
 */
public enum JobMode
{
   NOW       (false, "NOW"),
   RECURRENT (true,  "RECURRENT"),
   ONE_TIME  (true,  "ONE-TIME");
   
   /** Flag indicating if this job mode is timed. */
   private final boolean timed;
   
   /** The job mode. */
   private final String mode;
   
   /**
    * Initialize a new job type.
    *
    * @param    timed
    *           Flag indicating if this is a timed job or not.
    * @param    mode
    *           The job mode.
    */
   JobMode(boolean timed, String mode)
   {
      this.timed = timed;
      this.mode = mode;
   }

   /**
    * Check if this job mode is timed (i.e. it must be scheduled for execution at a certain 
    * time/date).
    * 
    * @return   See above.
    */
   public boolean isTimed()
   {
      return this.timed;
   }

   /**
    * Resolve the job mode from the given string.
    * 
    * @param    mode
    *           The job mode.
    *           
    * @return   One of the {@link JobMode} values or {@code null} if it can not be resolved.
    */
   public static JobMode resolveJobMode(String mode)
   {
      for (JobMode jobMode : JobMode.values())
      {
         if (jobMode.mode.equalsIgnoreCase(mode))
         {
            return jobMode;
         }
      }

      return null;
   }
}