/*
** Module   : JobProcessor.java
** Abstract : Concrete job launcher, which listens for jobs posted via the timer thread and
**            executes them.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140206 First version.
*/

package com.goldencode.p2j.scheduler;

import java.util.*;

import com.goldencode.p2j.net.*;
import com.goldencode.p2j.security.*;
import com.goldencode.p2j.util.*;

/**
 * Specialized class which listens for incoming jobs (sent by the {@link Scheduler#timer}.  
 * This class runs in a separate thread, under the server's context.
 */
class JobProcessor
implements Runnable
{
   /** The list with the pending jobs. */
   private final LinkedList<Runnable> pendingJobs = new LinkedList<>();
   
   /**
    * Initialize the scheduler and schedule the initial jobs.  Then, start listening for new
    * jobs being posted to the {@link #pendingJobs queue} by the {@link Scheduler#timer}.
    */
   @Override
   public void run()
   {
      Scheduler.initialize();
      
      // before starting, wait for the server to start listening
      try
      {
         SessionManager.get().waitUntilReady();
      }
      catch (InterruptedException e)
      {
         return;
      }
      
      // add initial jobs
      synchronized (Scheduler.jobs)
      {
         for (Job job : Scheduler.jobs.values())
         {
            if (!job.isEnabled())
            {
               // skip disabled jobs
               continue;
            }
            
            Scheduler.scheduleJob(job.getName(), null);
         }
      }
      
      Runnable core = null;

      // listen for incoming jobs
      while (true)
      {
         synchronized (pendingJobs)
         {
            try
            {
               pendingJobs.wait();
            }
            catch (InterruptedException e)
            {
               // someone interrupted me, exit
               break;
            }
            
            core = (pendingJobs.isEmpty()) ? null : pendingJobs.removeFirst();
         }
         
         if (core != null)
         {
            // execute this in a different thread, but with the server's context
            Thread t = new AssociatedThread(core);
            String tname = LogHelper.createThreadName(String.format(core.toString()));
            t.setDaemon(true);
            t.setName(tname);
            t.start();
         }
      }
   }
   
   /**
    * Execute the given job.
    * 
    * @param    core
    *           The {@link Runnable} instance to execute the job.
    */
   public void executeJob(Runnable core)
   {
      synchronized (pendingJobs)
      {
         pendingJobs.addLast(core);
         pendingJobs.notify();
      }
   }
}