/*
** Module   : JobTask.java
** Abstract : Implements tasks to be passed to the java timer.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140206 First version.
*/

package com.goldencode.p2j.scheduler;

import java.util.*;
import java.util.logging.*;

import com.goldencode.p2j.util.*;

/**
 * Implements tasks to be passed to the java {@link Scheduler#timer}.
 * <p>
 * When a job is executed, it will delegate the job execution to the 
 * {@link Scheduler#processor job processor}, which will execute the job using the server's
 * context. 
 */
class JobTask
extends TimerTask
{
   /** Logger. */
   private static final Logger LOG = LogHelper.getLogger(JobTask.class);

   /** The job name. */
   private final String jobName;
   
   /**
    * Create a new job task to be registered with the {@link Scheduler#timer}.
    */
   public JobTask(String jobName)
   {
      this.jobName = jobName.toLowerCase();
   }

   /**
    * Execute a the job with the given {@link #jobName name}.
    */
   @Override
   public void run()
   {
      // we are executing, remove it from the map
      synchronized (Scheduler.jobRegistry)
      {
         Scheduler.jobRegistry.remove(jobName);
      }

      // get the configured job, reschedule it (if needed) and execute it
      synchronized (Scheduler.jobs)
      {
         Job job = Scheduler.getJob(jobName);
         
         if (!job.isEnabled())
         {
            // the job is not enabled.
            if (LOG.isLoggable(Level.WARNING))
            {
               String msg = String.format("Can not reschedule: job %s is disabled.", jobName);
               LOG.logp(Level.WARNING, "Scheduler$JobTask", "run", LogHelper.generate(msg));
               return;
            }
         }

         // schedule it again (depending on the mode)
         Scheduler.scheduleJob(jobName, this.scheduledExecutionTime());
         
         if (LOG.isLoggable(Level.FINE))
         {
            String msg = String.format("Executing job %s at %tc.",
                                       jobName, this.scheduledExecutionTime());
            LOG.logp(Level.FINE, "Scheduler$JobTask", "run", LogHelper.generate(msg));
         }
         
         // get the core executor
         Runnable core = Scheduler.getRunner(job);
         
         Scheduler.processor.executeJob(core);
      }
   }
}