/*
** Module   : SSLCertFactory.java
** Abstract : defines APIs to generate SSL certificates and their private keys. 
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140207 First version.
*/

package com.goldencode.p2j.security;

import java.math.*;
import java.security.*;
import java.util.*;

/**
 * Define APIs to generate SSL certificates and their private keys.
 * <p>
 * Before starting to generate certificates, a root CA needs to be generated.  This will be used
 * for signing all the issued certificates.
 */
public abstract class SSLCertFactory
{
   /** The public key exponent. */
   protected BigInteger exponent;
   
   /** The private key size: 1024 bytes or better.*/
   protected int keyStrength;
   
   /**
    * Decrypt a private key which was previously encrypted with the given password.
    * 
    * @param    encrypted
    *           The bytes representing the encrypted private key.
    * @param    password
    *           The encryption password.
    * 
    * @return   The decrypted {@link PrivateKey private key}.
    * 
    * @throws   SSLCertGenException
    *           If the private key could not be decrypted.
    */
   public abstract PrivateKey decryptPrivateKey(byte[] encrypted, String password)
   throws SSLCertGenException;

   /**
    * Encrypt the given key using the provided password.
    * 
    * @param    key
    *           The key to be encrypted.
    * @param    password
    *           The encryption password.
    *           
    * @return   A byte array with the encrypted key.
    * 
    * @throws   SSLCertGenException
    *           If the key could not be encrypted.
    */
   public abstract byte[] encryptPrivateKey(Key key, String password)
   throws SSLCertGenException;

   /**
    * Generate a certificate and sign it with the already generated root CA.
    * <p>
    * The encrypted private key will be saved in the specified {@code certKeyStore}; the encrypt
    * password will be returned by this API.
    * <p>
    * The public certificate will be saved in the specified {@code certStore}.
    * 
    * @param    alias
    *           The certificate alias, used to store the private key and certificate.
    * @param    validity
    *           The certificate validity, in years.
    * @param    commonName
    *           The certificate's common name (CN).
    * @param    fieldMap
    *           A map with additional subject attributes.
    * @param    certStore
    *           The store where to save the certificate.
    * @param    certKeyStore
    *           The store where to save the private key.
    *
    * @return   The password used to encrypt the private key in the store.
    * 
    * @throws   SSLCertGenException
    *           If the root CA is not yet generated or the certificate could not be generated.
    */
   public abstract String generateCertificate(String              alias,
                                              int                 validity,
                                              String              commonName,
                                              Map<String, String> fieldMap,
                                              KeyStore            certStore,
                                              KeyStore            certKeyStore)
   throws SSLCertGenException;

   /**
    * Generate a root CA certificate, which will be used to sign all the issues certificates.
    * <p>
    * The encrypted private key will be saved in the specified {@code certKeyStore}; the encrypt
    * password will be returned by this API.
    * <p>
    * The public root CA certificate will be saved in the specified {@code certStore}.
    * 
    * @param    alias
    *           The certificate alias, used to store the private key and certificate.
    * @param    validity
    *           The certificate validity, in years.
    * @param    commonName
    *           The certificate's common name (CN).
    * @param    fieldMap
    *           A map with additional subject attributes.
    * @param    certStore
    *           The store where to save the certificate.
    * @param    certKeyStore
    *           The store where to save the private key.
    *
    * @return   The password used to encrypt the private key in the store.
    * 
    * @throws   SSLCertGenException
    *           If the root CA is not yet generated or the certificate could not be generated.
    */
   public abstract String generateRootCA(String              alias,
                                         int                 validity,
                                         String              commonName,
                                         Map<String, String> fieldMap,
                                         KeyStore            certStore,
                                         KeyStore            certKeyStore)
   throws SSLCertGenException;
   
   /**
    * Generate a self-signed certificate. This may be used as a root CA.
    * <p>
    * The encrypted private key will be saved in the specified {@code certKeyStore}; the encrypt
    * password will be returned by this API.
    * <p>
    * The public certificate will be saved in the specified {@code certStore}.
    * 
    * @param    alias
    *           The certificate alias, used to store the private key and certificate.
    * @param    certificateAuthority
    *           Flag indicating if the generated self-signed certificate will be used as the root
    *           CA.
    * @param    validity
    *           The certificate validity, in years.
    * @param    commonName
    *           The certificate's common name (CN).
    * @param    fieldMap
    *           A map with additional subject attributes.
    * @param    certStore
    *           The store where to save the certificate.
    * @param    certKeyStore
    *           The store where to save the private key.
    *
    * @return   The password used to encrypt the private key in the store.
    * 
    * @throws   SSLCertGenException
    *           If the root CA is not yet generated or the certificate could not be generated.
    */
   public abstract String generateSelfSignedCertificate(String              alias,
                                                        boolean             certificateAuthority,
                                                        int                 validity,
                                                        String              commonName,
                                                        Map<String, String> fieldMap,
                                                        KeyStore            certStore,
                                                        KeyStore            certKeyStore)
   throws SSLCertGenException;

   /**
    * Get the map with the mandatory subject attributes.  The key is the string representation of
    * this attribute, while the value will be the associated description. 
    * 
    * @return   See above.
    */
   public abstract Map<String, String> getMandatorySubjectFields();

   /**
    * Initialize this SSL certificate factory.
    * 
    * @param    keyStrength
    *           The private key size: 1024 bytes or better.  If {@code null}, defaults to 1024.
    * @param    exponent
    *           The public key exponent.  If {@code null}, defaults to 65537.
    *           WARNING: a wrong value may result in vulnerable SSL private keys and also 3rd party 
    *           software might not accept them. Use with care.
    * 
    * @throws   SSLCertGenException
    *           If the factory could not be instantiated.
    */
   public void init(Integer keyStrength, BigInteger exponent)
   throws SSLCertGenException
   {
      this.keyStrength = (keyStrength == null) ? 1024 : keyStrength;
      if (this.keyStrength < 1024)
      {
         final String msg = "The RSA private key size %d is smaller than 1024 bytes.";
         throw new SSLCertGenException(String.format(msg, this.keyStrength));
      }
   
      this.exponent = (exponent == null) ? new BigInteger("65537") : exponent;

      if (this.exponent.testBit(0) == false)
      {
         final String msg = "The RSA public exponent %s is even.";
         throw new SSLCertGenException(String.format(msg, this.exponent.toString()));
      }
   }
}
