/*
** Module   : RandomWordGenerator.java
** Abstract : Helper class to generate a pseudorandom word.
**                       
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 ECF 20131006 Created initial version.
** 002 CA  20140207 Added API to generate a random word using default values.
*/

package com.goldencode.util;

/**
 * Simple helper to create a random word, suitable for use as a password or other security token.
 */
public final class RandomWordGenerator
{
   /** Default random generator length. */
   public  static final int GEN_LENGTH = 36; 

   /** Default random generator digits */
   public static final int GEN_DIGITS = 8; 
   
   /** Default random generator symbols */   
   public static final int GEN_SYMBOLS = 4; 

   /**
    * Create a pseudorandom word using the default length, number of digits and symbol characters.
    * The remaining characters, if any, will be alphabetical (random mix of upper and lower case).
    * 
    * @return  A pseudorandom word or empty string.
    * 
    * @throws  IllegalArgumentException
    *          if the specified number of digits and symbols combined exceeds the specified
    *          length.
    */
   public static String create()
   {
      return create(GEN_LENGTH, GEN_DIGITS, GEN_SYMBOLS);
   }

   /**
    * Create a pseudorandom word of the specified length, containing the specified number of digit
    * and symbol characters. The remaining characters, if any, will be alphabetical (random mix of
    * upper and lower case).
    * 
    * @param   length
    *          Length of the word;  must be nonnegative and greater than or equal to the sum of
    *          <code>digits</code> and <code>symbols</code>.
    * @param   digits
    *          Number of digit characters required;  will be ignored if less than 0.
    * @param   symbols
    *          Number of symbol characters required;  will be ignored if less than 0.
    * 
    * @return  A pseudorandom word or empty string.
    * 
    * @throws  IllegalArgumentException
    *          if the specified number of digits and symbols combined exceeds the specified
    *          length.
    */
   public static String create(int length, int digits, int symbols)
   {
      digits = Math.max(0, digits);
      symbols = Math.max(0, symbols);
      int alphas = length - digits - symbols;
      
      if (alphas < 0)
      {
         throw new IllegalArgumentException("Invalid length: " + length);
      }
      
      char[] buf = new char[length];
      
      if (digits > 0)
      {
         fillRandomSlots(buf, digits, "0123456789");
      }
      
      if (symbols > 0)
      {
         fillRandomSlots(buf, symbols, "~!@#$%^&*()_+-=<>?`");
      }
      
      if (digits + symbols < length)
      {
         fillRandomSlots(buf, alphas, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
      }
      
      return (new String(buf));
   }
   
   /**
    * Using characters from the specified pool, randomly chosen, fill randomly chosen elements in
    * the given character buffer.
    * <p>
    * Implementation note: this algorithm is biased toward shorter words (i.e., tens to hundreds
    * of characters). Performance degrades non-linearly with larger word lengths.
    * 
    * @param   buffer
    *          Buffer to populate.
    * @param   number
    *          Number of fill characters to generate.
    * @param   pool
    *          Pool of potential candidate characters from which to choose.
    * 
    * @throws  IllegalArgumentException
    *          if <code>number</code> exceeds the buffer length.
    */
   private static void fillRandomSlots(char[] buffer, int number, String pool)
   {
      int len = buffer.length;
      if (number > len)
      {
         throw new IllegalArgumentException(
            "Requested number of fill characters cannot exceed buffer length");
      }
      
      int poolSize = pool.length();
      int rem = number;
      
      while (rem > 0)
      {
         int i = (int) (Math.random() * len);
         if (buffer[i] != 0)
            continue;
         
         int j = (int) (Math.random() * poolSize);
         buffer[i] = pool.charAt(j);
         
         rem--;
      }
   }
   
   /**
    * Command line entry point to this class.
    * 
    * @param   args
    *          <ol>
    *          <li><code>length</code> - length of word to be generated
    *          <li><code>digits</code> - number of digit characters to include in word
    *          <li><code>symbols</code> - number of symbol characters to include in word
    *          </ol>
    */
   public static void main(String[] args)
   {
      try
      {
         if (args.length != 3)
         {
            System.out.println("Usage: RandomWordGenerator <length> <digits> <symbols>");
            System.exit(-1);
         }
         
         int length = Integer.parseInt(args[0]);
         int digits = Integer.parseInt(args[1]);
         int symbols = Integer.parseInt(args[2]);
         
         String word = create(length, digits, symbols);
         
         System.out.println(word);
      }
      catch (Exception exc)
      {
         exc.printStackTrace();
      }
   }

}
