/*
** Module   : spawn.c
** Abstract : a simple tool able to spawn a process on an existing user account.
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description---------------------------------
** 001 MAG 20131127 First version.
** 002 MAG 20140205 Fix PTY (pseudo terminal) assignment bug.
** 003 CA  20140212 Added no password authentication: uses a secure P2J connection and temporary
**                  credentials to authenticate the request.
*/

#define _GNU_SOURCE
#include <syslog.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <unistd.h>
#include <string.h>
#include <pwd.h>
#include <shadow.h>
#include <signal.h>
#include <grp.h>
//#define _XOPEN_SOURCE 600
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <jni.h>

#ifdef PAM
#include <security/pam_appl.h>
#define PAM_SERVICE_NAME "common-auth"
#else
#include <crypt.h>
#endif

#define P2J_JAR    "p2j.jar"
#define KEY_STORE  "srv-certs.store"

#define MIN_ARGS_PASSWORD 4
#define MIN_ARGS_NO_PASSWORD 5
#define MIN_ARGS_COMMAND 3
#define EXIT_SUCCESS 0

#define ERR_SPAWN_NO_ROOT_PERMISSION -1
#define ERR_SPAWN_NO_ARGS            -2
#define ERR_SPAWN_NO_PASSWORD        -3
#define ERR_SPAWN_POSIX_OPENPT       -4
#define ERR_SPAWN_GRANPT             -5
#define ERR_SPAWN_UNLOCKPT           -6
#define ERR_SPAWN_OPEN_SLAVE         -7
#define ERR_SPAWN_IOCTL              -8
#define ERR_SPAWN_SETGID             -9
#define ERR_SPAWN_SETEGID           -10
#define ERR_SPAWN_INITGROUPS        -11
#define ERR_SPAWN_SETUID            -12
#define ERR_SPAWN_SETEUID           -13
#define ERR_SPAWN_SETSID            -14
#define ERR_SPAWN_CHDIR             -15
#define ERR_SPAWN_SETPGID           -16
#define ERR_SPAWN_FORK_LEADER       -17
#define ERR_SPAWN_FORK_COMMAND      -18
#define ERR_SPAWN_CHDIR_WD          -19

#define ERR_JNI_JVM_CREATE             -100
#define ERR_JNI_P2J_ENTRY_POINT_CLASS  -101
#define ERR_JNI_P2J_ENTRY_POINT_METHOD -102
#define ERR_JNI_REMOTE_SERVER_PORT     -103
#define ERR_JNI_REMOTE_SERVER_HOST     -104
#define ERR_JNI_REMOTE_SERVER_ALIAS    -105
#define ERR_JNI_REMOTE_SERVER_UUID     -106
#define ERR_JNI_REMOTE_COMMAND         -107
#define ERR_JNI_JVM_DESTROY            -108

#define AUTH_OK           0
#define AUTH_ERR_USER     1
#define AUTH_ERR_PASSWORD 2
#define AUTH_ERR_ROOT     3
#define AUTH_ERR_PAM      4

#define DEBUG_JVM 1

/* Global data */
struct passwd *pw;

/* File descriptors */
int fdm, fds;

/**
* Use syslog to audit the execution of child processes.
*
* @param   code
*          Exit code.
* @param   method
*          Method name where error is throw.
*
* @return  Exit code
*/
int audit(int code, const char *method)
{
   openlog("spawn", LOG_PID, LOG_USER);
   syslog(LOG_ERR, "Error:%d method:%s (%m)\n", code, method);
   closelog();
   return code;
}

/**
* Assign a controlling terminal to a process.
*
* @return  0 - Success
*          1 - Error
*/
int assign_pty()
{
   int result;

   /* Open a pseudo-terminal device */
   fdm = posix_openpt(O_RDWR);
   if (fdm < 0)
   {
      return audit(ERR_SPAWN_POSIX_OPENPT, "posix_openpt");
   }

   /* Grant access to the slave pseudo-terminal device */
   result = grantpt(fdm);
   if (result != 0)
   {
      return audit(ERR_SPAWN_GRANPT, "grantpt");
   }

   /*  Unlock a pseudo-terminal master/slave pair */
   result = unlockpt(fdm);
   if (result != 0)
   {
      return audit(ERR_SPAWN_UNLOCKPT, "unlockpt");
   }

   /* open slave pseudo-terminal device */
   fds = open(ptsname(fdm), O_RDWR);
   if (fds < 0)
   {
      return audit(ERR_SPAWN_OPEN_SLAVE, "open(ptsname)");
   }

   /* Use the slave as TTY */
   if (ioctl(fds, TIOCSCTTY, (char *)1) == -1)
   {
      return audit(ERR_SPAWN_IOCTL, "ioctl");
   }

   return EXIT_SUCCESS;
}

/**
* Execute a command on a specified user account.
* A child process is created via fork().
* The child process uid and gid are set as the new user uid and gid.
* Create a new session for child. The child became a session leader.
* Assign a controlling terminal to session leader.
* The child process environment is set from the new user account.
* Create a child process to execute the command.
*
* @param   command
*          Command to execute
* @param   argv
*          Command arguments.
* @param   workdir
*          Process working directory.
*
* @return  Process exit status.
*/
int spawn(const char* command, char* const argv[], const char* workdir)
{
   pid_t child_pid, proc_pid;
   int status, result;

   /* Creates a child process */
   child_pid = fork();

   if (child_pid == 0)
   {
      /* Set gid */
      if (setgid(pw->pw_gid) < 0  || getgid() != pw->pw_gid)
      {
         exit(audit(ERR_SPAWN_SETGID, "setgid"));
      }

      /* Set egit*/
      if (setegid(pw->pw_gid) < 0 || getegid() != pw->pw_gid)
      {
         exit(audit(ERR_SPAWN_SETEGID, "setegid"));
      }

      /* Initialize the supplementary group access list */
      if (initgroups(pw->pw_name, pw->pw_gid) < 0)
      {
         exit(audit(ERR_SPAWN_INITGROUPS, "initgroups"));
      }

      /* Set uid */
      if (setuid(pw->pw_uid) < 0 || getuid() != pw->pw_uid)
      {
         exit(audit(ERR_SPAWN_SETUID, "setuid"));
      }

      /* Set euid */
      if (seteuid(pw->pw_uid) < 0 || geteuid() != pw->pw_uid)
      {
         exit(audit(ERR_SPAWN_SETEUID, "seteuid"));
      }

      /* Creates a session and sets the process group ID */
      if(setsid() < 0)
      {
         exit(audit(ERR_SPAWN_SETSID, "setsid"));
      }

      /* Assign controlling terminal */
      result = assign_pty();
      if (result != EXIT_SUCCESS)
      {
         exit(audit(result, "assign_pty"));
      }

      /* Set user environment */
      char* path = strdup(getenv("PATH"));
      setenv("HOME",pw->pw_dir,1);
      setenv("USER",pw->pw_name,1);
      setenv("LOGNAME",pw->pw_name,1);
      setenv("LOGIN",pw->pw_name,1); /*< Historical; only strictly needed on AIX */
      setenv("SHELL",pw->pw_shell[0] ? pw->pw_shell : "/bin/sh",1);
      setenv("PATH",path,1);
      free(path);

      /* Change the current working directory */
      if (chdir(getenv("HOME")) < 0)
      {
         if (chdir("/") < 0)
         {
            exit(audit(ERR_SPAWN_CHDIR, "chdir($HOME)"));
         }
      }

      /* Spawn a child process */
      proc_pid = fork();

      if (proc_pid == 0)
      {
         /* Close master pty */
         close(fdm);

         /* Set stdin, stdout and stderr */
         dup2(fds, STDIN_FILENO); // PTY becomes standard input (0)
         dup2(fds, STDOUT_FILENO); // PTY becomes standard output (1)
         dup2(fds, STDERR_FILENO); // PTY becomes standard error (2)
      
         /* Cancel certain signals */
         signal(SIGCHLD,SIG_DFL); /* A child process dies */
         signal(SIGTSTP,SIG_IGN); /* Various TTY signals */
         signal(SIGTTOU,SIG_IGN);
         signal(SIGTTIN,SIG_IGN);
         signal(SIGHUP, SIG_IGN); /* Ignore hangup signal */
         signal(SIGTERM,SIG_DFL); /* Die on SIGTERM */

         /* Set process group ID */
         if (setpgid(0, child_pid) < 0)
         {
            exit(audit(ERR_SPAWN_SETPGID, "setpgid"));
         }

         /* Set process working directory */
         if (chdir(workdir))
         {
            exit(audit(ERR_SPAWN_CHDIR_WD, "chdir(workdir)"));
         }

         /* Execute command */
         exit(execvp(command, argv));
      }
      else if (proc_pid < 0)
      {
         /* On fork() fails */
         exit(audit(ERR_SPAWN_FORK_COMMAND, "fork"));
      }

      /* Close slave pty */
      close(fds);

      /* Wait for child to exit*/
      wait(&status);

      /* Exit */
      exit(status);
   }
   else if (child_pid < 0)
   {
      /* On fork() fails */
      exit(audit(ERR_SPAWN_FORK_LEADER, "fork"));
   }

   return EXIT_SUCCESS;
}

#ifdef PAM
struct pam_response *reply;

/**
* PAM conversation function.
* This is a minimal implementation.
*
* @param   num_msg
*          The number of messages that are being passed to the function.
* @param   msg
*          A pointer to the buffer that holds the messages from the user.
* @param   resp
*          A pointer to the buffer that holds the responses to the user.
* @param   appdata_ptr
*          Pointer to the application data.
*
* @return  PAM_SUCCESS      - success.
*/
int function_conversation(int num_msg,
                          const struct pam_message **msg,
                          struct pam_response **resp,
                          void *appdata_ptr)
{
   *resp = reply;
   return PAM_SUCCESS;
}

/**
* PAM authentication.
*
* @param   username
*          User name.
* @param   password
*          Use password.
*
* @return  AUTH_OK         - success.
*          AUTH_ERR_USER    - user does not exists.
*          AUTH_ERR_PAM     - on other errors.
*/
int check_user(const char *username, char *password)
{
   int retval;
   const char *user;
   pam_handle_t *local_auth_handle = NULL; // this gets set by pam_start
   const struct pam_conv local_conversation = { function_conversation, NULL };

   /* The pam_start function creates the PAM context and initiates the PAM transaction. */
   retval = pam_start(PAM_SERVICE_NAME, username, &local_conversation, &local_auth_handle);

   if (retval != PAM_SUCCESS)
   {
      return audit(AUTH_ERR_PAM, "pam_start");
   }

   if (password != NULL)
   {
      reply = (struct pam_response *)malloc(sizeof(struct pam_response));
   
      reply->resp = password;
      reply->resp_retcode = 0;
   
      /* The pam_authenticate function is used to authenticate the user. */
      retval = pam_authenticate(local_auth_handle, 0);
   
      if (retval != PAM_SUCCESS)
      {
         switch (retval)
         {
            case PAM_USER_UNKNOWN:
               return audit(AUTH_ERR_USER, "pam_authenticate");
            default:
               return audit(AUTH_ERR_PAM, "pam_authenticate");
         }
      }
   }

   /* Get mapped user name; PAM may have changed it */
   retval = pam_get_item(local_auth_handle, PAM_USER, (const void **)&user);
   if (retval != PAM_SUCCESS)
   {
      return audit(AUTH_ERR_USER, "pam_get_item");
   }

   /* Get a pointer to a structure containing the broken-out fields
   of the record in the password database (e.g., the local password
   file /etc/passwd, NIS, and LDAP) that matches the username name. */
   pw = getpwnam(user);
   endpwent();

   if (!pw) return audit(AUTH_ERR_USER, "getpwnam"); //user doesn't really exist

   if (pw->pw_uid == 0)
   {
      return AUTH_ERR_ROOT;   // user is root
   }

   /* The pam_end function terminates the PAM transaction and is the
      last function an application should call in the PAM context. */
   retval = pam_end(local_auth_handle, retval);

   if (retval != PAM_SUCCESS)
   {
      return audit(AUTH_ERR_PAM, "pam_end");
   }

   return AUTH_OK;
}

#else

/**
* Get user account informations including uid, gid and password.
* If the user exists check for password match.
* This is basically an authentication action.
* The user root is not allowed.
*
* @param   username
*          User name.
* @param   password
*          Use password.
*
* @return  AUTH_OK         - success.
*          AUTH_ERR_USER    - user does not exists.
*          AUTH_ERR_PASSWORD - password does not match.
*          AUTH_ERR_ROOT    - user is root.
*/
int check_user(const char* username, const char* password)
{
   struct spwd *sp;
   char *encrypted, *correct;

   /* Get a pointer to a structure containing the broken-out fields
   of the record in the password database (e.g., the local password
   file /etc/passwd, NIS, and LDAP) that matches the username name. */
   pw = getpwnam(username);
   endpwent();

   if (!pw) return audit(AUTH_ERR_USER, "getpwnam"); //user doesn't really exist

   if (pw->pw_uid == 0)
   {
      return AUTH_ERR_ROOT;   // user is root
   }

   if (password != NULL)
   {
      /* Get a pointer to a structure containing the broken-out fields
      of the record in the shadow password database that matches the username name. */
      sp = getspnam(pw->pw_name);
      endspent();
   
      correct = sp ? sp->sp_pwdp : pw->pw_passwd;
      encrypted = crypt(password, correct);
   
      // bad pw=2, success=0
      return strcmp(encrypted, correct) ? AUTH_ERR_PASSWORD : AUTH_OK;
   }
   else
   {
      return AUTH_OK;
   }
}

#endif

/**
* Check executable file attributes.
*
* @param   path
*          path to executable file.
* @param   permissions
*          The file's permissions
*
* @return  0 - OK.
*          1 - No root permissions.
*/
int check_root(const char *path, int permissions)
{
   struct stat pstat;
   int result;

   result = stat(path, &pstat);

   if (!result)
   {
      result = (pstat.st_uid == 0) &&
               (pstat.st_gid == 0) &&
               (pstat.st_mode & ALLPERMS) == permissions ? 0 : 1;
   }
   else
   {
      audit(result, "stat");
   }

   return result;
}

/**
* Get the user password from stdin.
*
* @return  Password or NULL on error.
*/
char* password()
{
   int read;
   size_t len;
   char* pswd = NULL;

   /* Read a line form console */
   read = getline(&pswd, &len, stdin);

   if (read == -1)
   {
      return NULL;
   }
   else if (pswd[read-1] == '\n')
   {
      pswd[read-1] = 0;
   }

   return pswd;
}

/**
* Print help.
*
* @param   pname
*          The program's name.
*/
int help(char *pname)
{
   printf("Usage:\n");
   printf("- for password authentication: %s 1 <user> <workdir> <command> [args]\n", pname);
   printf("- for server authentication: %s 0 <secure-port> <host> <server-alias> <uuid>\n", pname);
   
   return EXIT_SUCCESS;
}

/**
* Launch an in-process JVM and connects to the specified server. Authentication is assumed passed
* only if a secure connection can be establiashed with the server and the server's certificate  
* matches one from the store.
* <p>
* If authentication passes, starts a new process with the real command.
*
* @param   cport
*          The P2J server's port.
* @param   chost
*          The P2J server's host.
* @param   calias
*          The certificate alias for the P2J server.
* @param   uuid
*          Unique identifier for this request; must be known by the target P2J server.
*/
int launchP2JClient(char cport[], char chost[], char calias[], char cuuid[])
{
   JavaVM *jvm;
   JNIEnv *env;

   int nopts;
   
   JavaVMOption options[9];
   options[0].optionString = "-Djava.class.path=./p2j.jar:.";
   options[1].optionString = "-Djava.lib.path=.";
   options[2].optionString = "-Xmx512m";
   options[3].optionString = "-XX:MaxPermSize=64m";
   options[4].optionString = "-Djava.awt.headless=true";
   options[5].optionString = "-Djava.compiler=NONE";
   nopts = 6;
   
   if (DEBUG_JVM)
   {
      options[6].optionString = "-Xdebug";
      options[7].optionString = "-Xnoagent";
      options[8].optionString = "-Xrunjdwp:transport=dt_socket,address=2999,server=y,suspend=y";
      nopts = 9;
   }
   
   JavaVMInitArgs vm_args; /* VM initialization arguments */
   vm_args.version   = JNI_VERSION_1_6;
   vm_args.nOptions  = nopts;
   vm_args.options   = options;
   vm_args.ignoreUnrecognized = 0;

   jint err = JNI_CreateJavaVM(&jvm, (void**)&env, &vm_args);
   if (err != JNI_OK)
   {
      exit(audit(ERR_JNI_JVM_CREATE, "JNI_CreateJavaVM"));
   }

   jclass cls = (*env)->FindClass(env, "com/goldencode/p2j/main/NativeSecureConnection");
   if (cls == NULL)
   {
      (*jvm)->DestroyJavaVM(jvm);

      exit(audit(ERR_JNI_P2J_ENTRY_POINT_CLASS, "FindClass(NativeSecureConnection)"));
   }
   
   char *signature = "(ILjava/lang/String;Ljava/lang/String;Ljava/lang/String;)[Ljava/lang/String;";
   jmethodID mid = (*env)->GetStaticMethodID(env, cls, "command", signature);
   if (mid == NULL)
   {
      (*jvm)->DestroyJavaVM(jvm);

      exit(audit(ERR_JNI_P2J_ENTRY_POINT_METHOD, "GetStaticMethodID(command)"));
   }
   
   int port = atoi(cport);
   
   jstring host = (*env)->NewStringUTF(env, chost);
   if (host == NULL)
   {
      (*jvm)->DestroyJavaVM(jvm);

      exit(audit(ERR_JNI_REMOTE_SERVER_HOST, "NewStringUTF(host)"));
   }

   jstring alias = (*env)->NewStringUTF(env, calias);
   if (alias == NULL)
   {
      (*jvm)->DestroyJavaVM(jvm);
      
      exit(audit(ERR_JNI_REMOTE_SERVER_ALIAS, "NewStringUTF(alias)"));
   }

   jstring uuid = (*env)->NewStringUTF(env, cuuid);
   if (uuid == NULL)
   {
      (*jvm)->DestroyJavaVM(jvm);

      exit(audit(ERR_JNI_REMOTE_SERVER_UUID, "NewStringUTF(uuid)"));
   }

   jobjectArray command = (*env)->CallStaticObjectMethod(env, (jclass) cls, (jmethodID) mid,
                                                    port, host, alias, uuid);
   
   if (command == NULL)
   {
      (*jvm)->DestroyJavaVM(jvm);
      
      exit(audit(ERR_JNI_REMOTE_COMMAND, "CallStaticObjectMethod(command)"));
   }

   /* extract the arguments */
   jsize jlength = (*env)->GetArrayLength(env, (jarray) command);
   int length = (int) jlength;
   
   /* make it 1-based, so it follows the structure of the command line arguments 
      (0 is program name); also, one more position is needed for execvp, which requires for the
      arguments to be null-terminated. */
   char **arguments = (char**) malloc((length + 2) * sizeof(char*));
   arguments[0] = NULL;
   arguments[length + 1] = NULL;
   
   int i;
   for (i = 0; i < (int) length; i++)
   {
      jstring jarg = (jstring) (*env)->GetObjectArrayElement(env, command, (jsize) i);
      jsize argLength = (*env)->GetStringUTFLength(env, jarg);
      const char *narg = (*env)->GetStringUTFChars(env, jarg, 0);

      int nlength = (int) argLength + 1;
      arguments[i + 1] = (char*) malloc(nlength * sizeof(char));
      memset(arguments[i + 1], '\0', nlength);
      strcpy(arguments[i + 1], narg);

      (*env)->ReleaseStringUTFChars(env, jarg, narg);
   }

   err = (*jvm)->DestroyJavaVM(jvm);
   if (err < 0)
   {
      exit(audit(ERR_JNI_JVM_DESTROY, "DestroyJavaVM"));
   }
   
   if (length < MIN_ARGS_COMMAND)
   {
      help("");
      exit(ERR_SPAWN_NO_ARGS);
   }
   else
   {
      /* Check user account credentials */
      int result = check_user(arguments[1], NULL);
   
      switch (result)
      {
         case AUTH_OK:
            result = spawn(arguments[MIN_ARGS_COMMAND],
                           &arguments[MIN_ARGS_COMMAND],
                           arguments[MIN_ARGS_COMMAND - 1]);
            break;
         case AUTH_ERR_USER:
            puts("Invalid user name.");
            break;
         case AUTH_ERR_PASSWORD:
            puts("Invalid password.");
            break;
         case AUTH_ERR_ROOT:
            puts("User root not allowed.");
            break;
      }
      
      return result;
   }

   return EXIT_SUCCESS;
}

/**
* Main function which allows both password and P2J server-style authentication.
* <p>
* For P2J server-style authentiation, the following files need to exist in current directory:
* - p2j.jar, which will automatically be added to classpath
* - srv-certs.store, a key store with the certificates for the valid P2J server which can invoke
* this utility.
*
* Link option: -lm -lcrypt -lpam -ljvm
* <p>
* After build do the following:
*    $ sudo chown root:root spawn
*    $ sudo chmod 4755 spawn
*    $ sudo chmod 0440 srv-certs.store
*    $ sudo chmod 0440 p2j.jar
*
* @param   argc
*          Number of command line arguments.
* @param   argv
*          Command line arguments.
*
* @return  Exit status.
*/
int main(int argc, char *argv[])
{
   int result;

   /* Check for root permissions */
   if (check_root(argv[0], 04755))
   {
      puts("No root access permissions.");
      exit(ERR_SPAWN_NO_ROOT_PERMISSION);
   }

   /* Print a short help */
   if (argc == 1)
   {
      help(argv[0]);
      exit(ERR_SPAWN_NO_ARGS);
   }

   char *smode = argv[1];
   if (strcmp(smode, "1") == 0)
   {
      if (argc < MIN_ARGS_PASSWORD)
      {
         help(argv[0]);
         exit(ERR_SPAWN_NO_ARGS);
      }

      /* Ask for password */
      char *psw = password();
   
      if (psw == NULL)
      {
         puts("No password.");
         exit(ERR_SPAWN_NO_PASSWORD);
      }

      /* Check user account credentials */
      result = check_user(argv[2], psw);
      
      /* Password no longer needed. */
      free(psw);

      switch (result)
      {
         case AUTH_OK:
            result = spawn(argv[MIN_ARGS_PASSWORD],

                           &argv[MIN_ARGS_PASSWORD],
                           argv[MIN_ARGS_PASSWORD - 1]);
            break;
         case AUTH_ERR_USER:
            puts("Invalid user name.");
            break;
         case AUTH_ERR_PASSWORD:
            puts("Invalid password.");
            break;
         case AUTH_ERR_ROOT:
            puts("User root not allowed.");
            break;
      }
   }
   else if (strcmp(smode, "0") == 0)
   {
      if (check_root(KEY_STORE, 00550))
      {
         puts("No root access permissions for srv-certs.store.");
         exit(ERR_SPAWN_NO_ROOT_PERMISSION);
      }
      if (check_root(P2J_JAR, 00550))
      {
         puts("No root access permissions for p2j.jar.");
         exit(ERR_SPAWN_NO_ROOT_PERMISSION);
      }
      
      if (argc < MIN_ARGS_NO_PASSWORD)
      {
         help(argv[0]);
         exit(ERR_SPAWN_NO_ARGS);
      }

      result = launchP2JClient(argv[2], argv[3], argv[4], argv[5]);
   }
   else
   {
      help(argv[0]);
      exit(ERR_SPAWN_NO_ARGS);
   }

   exit(result);
}
